import shutil

def setup_build_tools(project_name: str, selected_build_tool: str):
    
    if selected_build_tool == 'Make':
        shutil.copyfile('../core/build_tools/Make/Makefile', f'./{project_name}/Makefile')



from .create_project import create_project
from .setup_package_manager import setup_package_manager
from .setup_build_tools import setup_build_tools

__all__ = [
    'create_project',
    'setup_package_manager',
    'setup_build_tools'
]


from InquirerPy import get_style
from InquirerPy.base.control import Choice


CLI_STYLES = get_style({
    "questionmark": "#8BE9FD",        
    "answermark": "#50FA7B",          
    "answered_question": "#BD93F9",
    "question": "#BD93F9",
    "checkbox": "#8BE9FD",
    "pointer": "#8BE9FD",
})

CLI_SETTINGS = {
    'qmark': '?',
    'amark': '✔',
    'style': CLI_STYLES
}

PACKAGE_MANAGEMENT = [
    'pip-tools',
    Choice(value=None, name='None')
]

ENVIROMENT_SETUP_TOOLS = [
    'Make',
    Choice(value=None, name='None')
]

THIRD_PARTY_BASE_PACKAGES = []

THIRD_PARTY_DEV_PACKAGES = [
    'ipdb',
]

THIRD_PARTY_PROD_PACKAGES = []

THIRD_PARTY_PACKAGES = THIRD_PARTY_BASE_PACKAGES + THIRD_PARTY_DEV_PACKAGES + THIRD_PARTY_PROD_PACKAGES


import shutil
import subprocess
from subprocess import DEVNULL
import venv
import tempfile
from pathlib import Path
from config import THIRD_PARTY_DEV_PACKAGES, THIRD_PARTY_PROD_PACKAGES

def add_packages(name, packages):
    base_requirements_path = f'./{name}/requirements'

    for package in packages:
        requirements_path = base_requirements_path

        if package in THIRD_PARTY_DEV_PACKAGES:
            requirements_path += '/dev.in'
        elif package in THIRD_PARTY_PROD_PACKAGES:
            requirements_path += '/prod.in'
        else:
            requirements_path += '/base.in'

        with open(requirements_path, '+a') as file:
            file.write(f'{package}\n')

def setup_package_manager(name: str, package_management_tool: str, packages: list[str]):

    if package_management_tool == 'pip-tools':
        with tempfile.TemporaryDirectory() as temp_directory:
            temp_directory_path = Path(temp_directory)

            venv.EnvBuilder(with_pip=True).create(temp_directory_path)

            shutil.copytree('../core/requirements/pip-tools', f'./{name}/requirements')
            add_packages(name, packages)
            subprocess.run(
                ['pip', 'install', 'pip-tools'],
                stdout=DEVNULL,
                stderr=DEVNULL
            )

            subprocess.run(
                ['pip-compile', '-o', f'./{name}/requirements/requirements-dev.txt', f'./{name}/requirements/dev.in'],
                stdout=DEVNULL,
                stderr=DEVNULL
            )





import typer 
from InquirerPy import inquirer
from InquirerPy.validator import EmptyInputValidator
from rich.progress import Progress, SpinnerColumn, TextColumn

from utils import create_project, setup_package_manager, setup_build_tools
from config import CLI_SETTINGS, PACKAGE_MANAGEMENT, ENVIROMENT_SETUP_TOOLS, THIRD_PARTY_PACKAGES

build_app = typer.Typer()

@build_app.command('build')
def cli_build_project():

    name = inquirer.text(
        message="Project name:",
        validate=EmptyInputValidator(),
        **CLI_SETTINGS
    ).execute()

    
    build_tools = inquirer.select(
        message="Select package management tool:",
        choices=PACKAGE_MANAGEMENT,
        default="requirements.txt",
        **CLI_SETTINGS
    ).execute()

    enviroment_setup = inquirer.select(
        message="Select enviroment setup:",
        choices=ENVIROMENT_SETUP_TOOLS,
        default="Make",
        **CLI_SETTINGS
    ).execute()

    install_packages = inquirer.checkbox(
        message="select third party packages:",
        choices=THIRD_PARTY_PACKAGES,
        default="Make",
        **CLI_SETTINGS
    ).execute()

    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        progress.add_task(description="creating project...", total=None)
        create_project(name)
        setup_package_manager(name, build_tools, install_packages)
        setup_build_tools(name, enviroment_setup)


from pathlib import Path
import environ
import os

BASE_DIR = Path(__file__).resolve().parent.parent.parent

# Enviroment variable definitions

env = environ.Env()
env.read_env(os.path.join(BASE_DIR, '.env'))

DEBUG = env.bool('DEBUG', False)

SECRET_KEY = env('SECRET_KEY')

# Application definitions

DJANGO_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

THIRD_PARTY_APPS = []

CUSTOM_APPS = []

INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + CUSTOM_APPS

DJANGO_MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

THIRD_PARTY_MIDDLEWARE = []

CUSTOM_MIDDLEWARE = []

MIDDLEWARE = DJANGO_MIDDLEWARE + THIRD_PARTY_MIDDLEWARE + CUSTOM_MIDDLEWARE

ROOT_URLCONF = 'config.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'config.wsgi.application'


# Database

DATABASES = {
    'default': env.db()
}


# Password validation

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)

STATIC_URL = 'static/'

# Default primary key field type

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
